GLOVE_DIM = 100
USE_CHAR_LEVEL_EMBEDDING = True
USE_CASING_FEATURE = True
PRETRAIN_PATH = "pretrain/glove.6B.100d.txt"
ALL_CHARACTERS = " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[]^_`abcdefghijklmnopqrstuvwxyz{|}~"
