MODEL_MODE = 5

CHAR_EMBEDDING_NUM_VOCABS = 100
CHAR_EMBEDDING_VECTORIZE_DIM = 50

CNN_EMBEDDING_IN_CHANNELS = 1
CNN_EMBEDDING_OUT_CHANNELS = 50
CNN_EMBEDDING_KERNEL_SIZE = (3, 50)
CNN_EMBEDDING_PADDING_SIZE = (1, 0)

LSTM_EMBEDDING_HIDDEN_SIZE = 25
LSTM_EMBEDDING_NUM_LAYERS = 1
LSTM_EMBEDDING_BIDIRECTIONAL = True
CASING_NUM_VOCABS = 8
CASING_VECTORIZE_DIM = 50
LSTM_HIDDEN_SIZE = 275
P_DROPOUT = 0.5
CRF_TARGET_SIZE = 11
